

#pragma once

#include <JUI/Widget.hpp>

namespace JUI {
    /// The Rect Widget Class
    /// Renders a rectangle or "box" and also serves as a vesatile container for laying out other widgets.
    class Rect : public Widget {
    public: /// Constructors
        /// The default constructor.
        Rect();

        Rect(Widget* parent);

        ~Rect() override {}
    public:
        Event<Vector2> MouseEnter;
        Event<Vector2> MouseExit;
        Event<Vector2> MousePress;
        Event<Vector2> MouseRelease;

    public:
        void Draw() override;
        void Update(float delta) override;
        bool IsMouseInside() const;

        Color4 GetBGColor() const { return bg_color; }
        Color4 GetBorderColor() const { return border_color;}
        float GetBorderWidth() const { return border_width;}


        void SetBGColor(const Color4& col) { bg_color = col;}
        void SetBorderColor(const Color4& col) { border_color = col;}
        void SetBorderWidth(float w) {border_width = w;}


        void SetCornerRounding(float radius);
        void SetCornerRounding(float tlRadius, float trRadius, float blRadius, float brRadius);
        void SetCornerRoundingTL(float radius);
        void SetCornerRoundingTR(float radius);
        void SetCornerRoundingBL(float radius);
        void SetCornerRoundingBR(float radius);


        void SetBorderStyling(const Color4& color, float width) {
            SetBorderColor(color);
            SetBorderWidth(width);
        }
    protected:
        float border_width = 1.f;
        Color4 bg_color = {128,128,128, 255};
        Color4 border_color = {192, 192, 192, 0};
        bool clips_descendants = false; // Controls if child objects can render outside of their parent's rectangle bounds.
        float corner_rounding_radius = 0.f; // Curves the rectangle corners by N degrees.
    };
}
