#pragma once
#include <Event.h>
#include <string>
#include <vector>
#include <J3ML/LinearAlgebra.h>
#include <JUI/UDim2.hpp>
#include <JGL/Color3.h>
#include <JGL/Color4.h>

namespace JUI
{

    using namespace J3ML::Math;
    using namespace J3ML::LinearAlgebra;
    using namespace JGL;

    class Widget
    {
    public:
        Widget();
        Widget(Widget* parent);
        virtual ~Widget() {}
    public:
        Event<> Focused;
        Event<> Unfocused;
        Event<Widget *> DescendantAdded;
        Event<Widget *> DescendantRemoved;
        Event<Widget *, Widget*> AncestryChanged;
        Event<Widget *> ChildAdded;
        Event<Widget *> ChildRemoved;
        Event<Widget *> Destroying;
    public:
        Widget* Add(Widget* newChild);
        bool IsAncestorOf(Widget* w) const;
        bool IsDescendantOf(Widget* w) const;
        Widget* FindFirstChild(const std::string& name);
        std::vector<Widget*> GetDescendants();
        std::vector<Widget*> GetAncestors();
        std::vector<Widget*> GetChildren();
        virtual Vector2 GetAbsoluteSize() const;
        virtual Vector2 GetAbsolutePosition() const;
        virtual float GetAbsoluteRotation() const;
        Widget* GetParent() const;

        [[nodiscard]] UDim2 GetPosition() const;

        [[nodiscard]] UDim2 GetSize() const;

        void SetPosition(const UDim2 &pos);

        void SetSize(const UDim2 &s);

        void SetParent(Widget*);

        bool IsDescendantOf(Widget *ancestor);

        bool IsAncestorOf(Widget *descendant);

        [[nodiscard]] Vector2 GetAnchorPoint() const;

        [[nodiscard]] UDim GetPaddingLeft() const;
        [[nodiscard]] UDim GetPaddingTop() const;
        [[nodiscard]] UDim GetPaddingRight() const;
        [[nodiscard]] UDim GetPaddingBottom() const;

        void SetPaddingLeft(const UDim &pad_left);
        void SetPaddingTop(const UDim &pad_top);
        void SetPaddingRight(const UDim &pad_right);
        void SetPaddingBottom(const UDim &pad_bottom);

        void SetPadding(const UDim& left, const UDim& top, const UDim& right, const UDim& bottom);
        void SetPadding(const UDim& padding);

        [[nodiscard]] UDim GetMarginLeft() const;
        [[nodiscard]] UDim GetMarginTop() const;
        [[nodiscard]] UDim GetMarginRight() const;
        [[nodiscard]] UDim GetMarginBottom() const;

        void SetMarginLeft(const UDim &ml);
        void SetMarginTop(const UDim &mt);
        void SetMarginRight(const UDim &mr);
        void SetMarginBottom(const UDim &mb);

        void SetMargin(const UDim& left, const UDim& top, const UDim& right, const UDim& bottom);
        void SetMargin(const UDim& margin);


        [[nodiscard]] std::string GetName() const;
        void SetName(const std::string &new_name);

        [[nodiscard]] bool IsVisible() const;

        void SetVisible(bool enabled);

    public:
        virtual void Draw();
        virtual void Update(float delta);
        void DrawChildWidgets();
        void UpdateChildWidgets(float delta);
        Widget* GetFamilyTreeRoot() const;
    protected:
        UDim2 position;
        UDim2 size;
        Widget* parent = nullptr;
        std::vector<Widget*> children;
        float rotation = 0;
        std::string name;
        bool selected = false;
        UDim pad_left = 0_px;
        UDim pad_right = 0_px;
        UDim pad_top = 0_px;
        UDim pad_bottom = 0_px;
        UDim margin_left = 0_px;
        UDim margin_right = 0_px;
        UDim margin_top = 0_px;
        UDim margin_bottom = 0_px;
        Vector2 anchor_point = {0.f, 0.f};
        bool visible = true;
        Widget* next = nullptr;
        Widget* prev = nullptr;
    };
}
