/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file main.cpp
/// @desc Demo Program Entry Point
/// @edit 2024-07-11

#include <iostream>
#include <JGL/JGL.h>
#include <JUI/Rect.hpp>
#include <JUI/Scene.hpp>
#include <rewindow/types/window.h>
#include <jlog/jlog.hpp>


class JUIDevelopmentTestWindow : public ReWindow::RWindow {
public:
    void initGL() {
        gladLoadGL();
        JGL::Update(getSize());
        JGL::InitTextEngine();
        glClearColor(0.f, 0.f, 0.f, 0.f);
    }

    JUIDevelopmentTestWindow(const std::string& title, int w, int h) : ReWindow::RWindow(title, w, h) {}
    void OnRefresh(float elapsed) override {
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        JGL::Update(getSize());

        //JGL::J2D::Begin();
            //JGL::J2D::FillRect(JGL::Color3(255,0,0), {0,0}, {50, 50});
        //JGL::J2D::End();
    }

    //bool OnResizeRequest(const ReWindow::WindowResizeRequestEvent &e) override {}
    JUIDevelopmentTestWindow() : ReWindow::RWindow() {}
};

JUI::Scene* CreateScene() {
    using namespace JUI;

    Scene* root = new Scene();

    Rect* element = new Rect(root);
    element->SetName("jimbob");
    element->SetBGColor({0,255,0});
    auto size = UDim2(64, 64, 0, 0);
    element->SetSize({0, 0, 1.f, 1.f});

    return root;
}
int main()
{

    auto* scene = CreateScene();

    JUI::Rect element = scene->FindFirstChild("jimbob");

    auto* window = new JUIDevelopmentTestWindow("Test Window", 600, 480);
    window->setRenderer(RenderingAPI::OPENGL);
    window->Open();
    window->initGL();

    window->setFullscreen(false);
    window->setVsyncEnabled(false);
    window->setResizable(true);

    while (window->isAlive()) {
        window->pollEvents();
        window->refresh();
        scene->SetViewportSize(window->getWidth(), window->getHeight());
        Vector2 winsz = window->getSize();
        int wwidth = window->getWidth();
        int wheight = window->getHeight();
        //element.SetSize({wwidth, wheight, winsz.x, winsz.y});
        //element.SetSize({0,0, winsz.x, winsz.y});
        //element.SetSize({wwidth, wheight, (float)wwidth, (float)wheight});
        //scene->SetViewportSize(window->getWidth(), window->getHeight());
        DEBUG(std::format("Window sz x: {} y: {}", wwidth, wheight));
        DEBUG(std::format("Window sz2 x: {} y: {}", winsz.x, winsz.y));
        JUI::UDim2 esz = element.GetSize();
        Vector2 escale = esz.GetScale();
        Vector2 epix = esz.GetPixels();
        DEBUG(std::format("Element scale x: {} y: {}", escale.x, escale.y));
        DEBUG(std::format("Element pix x: {} y: {}", epix.x, epix.y));
        scene->Update(0.f);
        scene->Draw();
        window->glSwapBuffers();
    }
    return 0;
}
